/*******************************************************************************

Highcharts Export Server

Copyright (c) 2016-2023, Highsoft

Licenced under the MIT licence.

Additionally a valid Highcharts license is required for use.

See LICENSE file in root for details.

*******************************************************************************/

import fetch from 'node-fetch';
import fs from 'fs';

// The request options
const configOptions = {
  method: 'POST',
  headers: {
    'Content-Type': 'application/json'
  },
  body: JSON.stringify({
    infile: {
      title: {
        text: 'Chart'
      },
      xAxis: {
        categories: ['Jan', 'Feb', 'Mar']
      },
      series: [
        {
          data: [29.9, 71.5, 106.4]
        }
      ]
    }
  })
};

const svgOptions = {
  method: 'POST',
  headers: {
    'Content-Type': 'application/json'
  },
  body: fs.readFileSync('./samples/http/request_svg.json', 'utf8') 
}

const requestsNumber = 50;

const pad = (str) => ('' + str).length < 2 ? '0' + str : str;

const stressTest = async (url, options) => {
  let totalTime = 0;

  // console.log(url);
  for (let i = 1; i <= requestsNumber; i++) {
    const startTime = new Date().getTime();

    // console.log( '  ' + pad(i) + '/' + requestsNumber);

    // Perform a request
    try {
    const fres = await fetch(url, options);
    const postTime = new Date().getTime() - startTime;

    if (!fres.ok) {
      console.error(url, 'Got', fres.status, 'Aborting.');
      return false;
    }

    totalTime += postTime;

    // Wait a bit
    await new Promise((resolve) => setTimeout(resolve, 5000));

    } catch (e) {
      console.error(e);
      return false;
    }
  }

  return totalTime / requestsNumber;
};

const doTest = async (options) => {  
  const res = {};
  for (const url of ['https://export-dev.highcharts.com', 'https://export.highcharts.com']) {
    res[url] = await stressTest(url, options);
  }

  console.log('');
  console.log('  Performed', requestsNumber, 'requests, with the following avarage processing:');
  console.log('   ', Object.keys(res).map(k => `${k}: ${res[k]}ms`).join('\n    '));
  console.log('');
}

const main = async () => {
  console.log('Config benchmark:');
  await doTest(configOptions);

  console.log('SVG input benchmark:');
  await doTest(svgOptions);
};

main().then(() => console.log('done!'));
